"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.IslandBase = void 0;
class IslandBase {
    constructor() {
        // eslint-disable-next-line no-magic-numbers
        this._generationNumber = 0;
        // eslint-disable-next-line no-magic-numbers
        this._offspringNumber = 0;
        this._initialized = false;
    }
    get generationNumber() {
        return this._generationNumber;
    }
    get offspringNumber() {
        return this._offspringNumber;
    }
    get initialized() {
        return this._initialized;
    }
    async reset() {
        this.population.init();
        await Promise.all(this.population.chromosomes.map(async (chromosome) => {
            await this.fitness.evaluate(chromosome);
        }));
        this._generationNumber = 0;
        this._offspringNumber = 0;
        this.performReset();
        this._initialized = true;
    }
    performReset() {
        // override if required
    }
    async step() {
        if (!this._initialized) {
            this.reset();
        }
        const { parents, population } = await this.selection.select(this.population.chromosomes);
        const offspring = await this.crossover.cross(parents, this.crossoverProbability);
        await Promise.all(offspring.map(async (chromosome) => {
            await this.mutation.mutate(chromosome, this.mutationProbability);
            await this.fitness.evaluate(chromosome);
        }));
        await Promise.all(parents.map(async (chromosome) => {
            await this.fitness.evaluate(chromosome);
        }));
        const newGeneration = await this.reinsertion.select(population, offspring, parents, this.population.size);
        await this.population.update(newGeneration);
        this._generationNumber++;
        this._offspringNumber += offspring.length;
        this.performStep();
    }
    performStep() {
        // override if required
    }
}
exports.IslandBase = IslandBase;
//# sourceMappingURL=IslandBase.js.map