"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MigrationBase = void 0;
class MigrationBase {
    constructor() {
        // eslint-disable-next-line no-magic-numbers
        this.prev = 0;
        // eslint-disable-next-line no-magic-numbers
        this.time = 0;
    }
    get rate() {
        // eslint-disable-next-line no-magic-numbers
        return 0.1;
    }
    get interval() {
        // eslint-disable-next-line no-magic-numbers
        return 10000;
    }
    init() {
        this.prev = 0;
        this.time = 0;
    }
    getCount(algorithm) {
        return algorithm.offspringNumber;
    }
    getDestinations(algorithm) {
        const islandNumber = algorithm.islands.length;
        // eslint-disable-next-line no-magic-numbers
        const time = this.time % (islandNumber - 1);
        // eslint-disable-next-line no-magic-numbers
        return [...Array(islandNumber).keys()].map(num => (num + 1 + time) % islandNumber);
    }
    async migrate(algorithm) {
        // eslint-disable-next-line no-magic-numbers
        if (algorithm.islands.length <= 1) {
            return;
        }
        const count = this.getCount(algorithm);
        if (count >= this.prev + this.interval) {
            this.prev = count;
            await this.performMigrate(algorithm);
            this.time++;
        }
    }
    static takeRandomChromosomes(chromosomes, count) {
        const results = [];
        // eslint-disable-next-line no-magic-numbers
        while (--count >= 0) {
            // eslint-disable-next-line no-magic-numbers
            results.push(...chromosomes.splice(Math.floor(Math.random() * chromosomes.length), 1));
        }
        return results;
    }
    getTakeCount(chromosomes) {
        return Math.ceil(chromosomes.length * this.rate);
    }
    static splitChromosomes(chromosomes, count) {
        const population = [...chromosomes];
        const emigrants = MigrationBase.takeRandomChromosomes(population, count);
        return { population, emigrants };
    }
    async performMigrate(algorithm) {
        const destination = this.getDestinations(algorithm);
        const islands = algorithm.islands.map(island => MigrationBase.splitChromosomes(island.population.chromosomes, this.getTakeCount(island.population.chromosomes)));
        islands.forEach((island, index) => {
            islands[destination[index]].population.push(...island.emigrants);
        });
        await Promise.all(algorithm.islands.map((island, index) => island.population.update(islands[index].population)));
    }
}
exports.MigrationBase = MigrationBase;
//# sourceMappingURL=MigrationBase.js.map